package ch.zhaw.pm2.multichat.protocol;

import java.io.EOFException;
import java.io.IOException;
import java.net.SocketException;
import java.util.Scanner;

public abstract class ConnectionHandler {
    private NetworkHandler.NetworkConnection<String> connection;

    // Data types used for the Chat Protocol
    private static final String DATA_TYPE_CONNECT = "CONNECT";
    private static final String DATA_TYPE_CONFIRM = "CONFIRM";
    private static final String DATA_TYPE_DISCONNECT = "DISCONNECT";
    private static final String DATA_TYPE_MESSAGE = "MESSAGE";
    private static final String DATA_TYPE_ERROR = "ERROR";

    public static final String USER_NONE = "";
    public static final String USER_ALL = "*";

    public enum State {
        NEW, CONFIRM_CONNECT, CONNECTED, CONFIRM_DISCONNECT, DISCONNECTED, ERROR;
    }

    public static String getDataTypeConnect() {
        return DATA_TYPE_CONNECT;
    }

    public static String getDataTypeConfirm() {
        return DATA_TYPE_CONFIRM;
    }

    public static String getDataTypeDisconnect() {
        return DATA_TYPE_DISCONNECT;
    }

    public static String getDataTypeMessage() {
        return DATA_TYPE_MESSAGE;
    }

    public static String getDataTypeError() {
        return DATA_TYPE_ERROR;
    }

    public NetworkHandler.NetworkConnection<String> getConnection() {
        return connection;
    }

    protected void setConnection(NetworkHandler.NetworkConnection<String> connection) {
        this.connection = connection;
    }

    protected void processData(Scanner scanner, StringBuilder sender, StringBuilder reciever, StringBuilder type, StringBuilder payload) throws ChatProtocolException {
        // parse data content
        if (scanner.hasNextLine()) {
            sender.append(scanner.nextLine());
        } else {
            throw new ChatProtocolException("No Sender found");
        }
        if (scanner.hasNextLine()) {
            reciever.append(scanner.nextLine());
        } else {
            throw new ChatProtocolException("No Reciever found");
        }
        if (scanner.hasNextLine()) {
            type.append(scanner.nextLine());
        } else {
            throw new ChatProtocolException("No Type found");
        }
        if (scanner.hasNextLine()) {
            payload.append(scanner.nextLine());
        }
    }

    protected void sendData(String sender, String receiver, String type, String payload) {
        if (connection.isAvailable()) {
            new StringBuilder();
            String data = new StringBuilder()
                    .append(sender+"\n")
                    .append(receiver+"\n")
                    .append(type+"\n")
                    .append(payload+"\n")
                    .toString();
            try {
                getConnection().send(data);
            } catch (SocketException e) {
                System.err.println("Connection closed: " + e.getMessage());
            } catch (EOFException e) {
                System.out.println("Connection terminated by remote");
            } catch(IOException e) {
                System.err.println("Communication error: " + e.getMessage());
            }
        }
    }
}
